@@ -10,8 +10,4 @@ from .image import *
 from .tabular import *
 from .mixed import *
 from .mixed_augmentation import *
-
-# Conditional import
-from ..imports import ExtraDependencies
-if ExtraDependencies.tsfresh:
-    from .features import *
\ No newline at end of file
+from .features import *
\ No newline at end of file
