@@ -16,6 +16,7 @@ from .ResCNN import *
 from .ResNet import *
 from .ResNetPlus import *
 from .RNN import *
+from .RNNAttention import *
 from .RNNPlus import *
 from .RNN_FCN import *
 from .RNN_FCNPlus import *
@@ -24,7 +25,9 @@ from .MINIROCKET import *
 from .ROCKET_Pytorch import *
 from .MINIROCKET_Pytorch import *
 from .MINIROCKETPlus_Pytorch import *
+from .MultiRocketPlus import *
 from .TransformerModel import *
+from .TransformerRNNPlus import *
 from .TST import *
 from .TSTPlus import *
 from .XceptionTime import *
